@@ -57,14 +57,17 @@ static void     ephy_embed_constructed (GObject *object);
 
 struct EphyEmbedPrivate
 {
+  GtkBox *top_widgets_vbox;
+  GtkScrolledWindow *scrolled_window;
   WebKitWebView *web_view;
   EphyHistory *history;
   GtkWidget *inspector_window;
-  char *load_failed_uri;
+  char *loading_uri;
   guint is_setting_zoom : 1;
+  GSList *destroy_on_transition_list;
 };
 
-G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_SCROLLED_WINDOW)
+G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_VBOX)
 
 static void
 uri_changed_cb (WebKitWebView *web_view,
@@ -169,11 +172,31 @@ resource_request_starting_cb (WebKitWebView *web_view,
   }
 }
 
+static void
+ephy_embed_destroy_top_widgets (EphyEmbed *embed)
+{
+  GSList *iter;
+
+  for (iter = embed->priv->destroy_on_transition_list; iter; iter = iter->next)
+    gtk_widget_destroy (GTK_WIDGET (iter->data));
+}
+
+static void
+remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
+{
+  GSList *list;
+
+  list = embed->priv->destroy_on_transition_list;
+  list = g_slist_remove (list, widget);
+  embed->priv->destroy_on_transition_list = list;
+}
+
 static void
 load_status_changed_cb (WebKitWebView *view,
                         GParamSpec *spec,
                         EphyEmbed *embed)
 {
+  EphyEmbedPrivate *priv = embed->priv;
   WebKitLoadStatus status = webkit_web_view_get_load_status (view);
 
   if (status == WEBKIT_LOAD_COMMITTED) {
@@ -182,15 +205,11 @@ load_status_changed_cb (WebKitWebView *view,
 
     uri = webkit_web_view_get_uri (view);
 
-    /* If the load failed for this URI, do nothing */
-    if (embed->priv->load_failed_uri &&
-        g_str_equal (embed->priv->load_failed_uri, uri)) {
-      g_free (embed->priv->load_failed_uri);
-      embed->priv->load_failed_uri = NULL;
+    ephy_embed_destroy_top_widgets (embed);
 
-      ephy_web_view_set_security_level (EPHY_WEB_VIEW (view), EPHY_WEB_VIEW_STATE_IS_UNKNOWN);
-
-      return;
+    if (g_strcmp0 (uri, priv->loading_uri) != 0) {
+      g_free (priv->loading_uri);
+      priv->loading_uri = g_strdup (uri);
     }
 
     ephy_web_view_location_changed (EPHY_WEB_VIEW (view),
@@ -214,7 +233,7 @@ load_status_changed_cb (WebKitWebView *view,
     ephy_web_view_set_security_level (EPHY_WEB_VIEW (view), security_level);
   } else if (status == WEBKIT_LOAD_PROVISIONAL || status == WEBKIT_LOAD_FINISHED) {
     EphyWebViewNetState estate = EPHY_WEB_VIEW_STATE_UNKNOWN;
-    const char *loading_uri = NULL;
+    char *loading_uri = NULL;
 
     if (status == WEBKIT_LOAD_PROVISIONAL) {
       WebKitWebFrame *frame;
@@ -224,7 +243,12 @@ load_status_changed_cb (WebKitWebView *view,
       frame = webkit_web_view_get_main_frame (view);
       source = webkit_web_frame_get_provisional_data_source (frame);
       request = webkit_web_data_source_get_initial_request (source);
-      loading_uri = webkit_network_request_get_uri (request);
+      loading_uri = g_strdup (webkit_network_request_get_uri (request));
+
+      /* We also store the URI we are currently loading here, because
+       * we will want to use it in WEBKIT_LOAD_FINISHED, because if a
+       * load fails we may never get to committed */
+      priv->loading_uri = g_strdup (loading_uri);
 
       estate = (EphyWebViewNetState) (estate |
                                       EPHY_WEB_VIEW_STATE_START |
@@ -234,17 +258,23 @@ load_status_changed_cb (WebKitWebView *view,
       
       g_signal_emit_by_name (EPHY_WEB_VIEW (view), "new-document-now", loading_uri);
     } else if (status == WEBKIT_LOAD_FINISHED) {
-      loading_uri = ephy_web_view_get_address (EPHY_WEB_VIEW (view));
+      loading_uri = priv->loading_uri;
+
+      /* Will be freed bellow */
+      priv->loading_uri = NULL;
 
       estate = (EphyWebViewNetState) (estate |
                                       EPHY_WEB_VIEW_STATE_STOP |
                                       EPHY_WEB_VIEW_STATE_IS_DOCUMENT |
                                       EPHY_WEB_VIEW_STATE_IS_NETWORK);
     }
-    
+
     ephy_web_view_update_from_net_state (EPHY_WEB_VIEW (view),
                                          loading_uri,
                                          (EphyWebViewNetState)estate);
+
+    g_free (loading_uri);
+
   }
 }
 
@@ -296,7 +326,7 @@ ephy_embed_grab_focus (GtkWidget *widget)
 {
   GtkWidget *child;
 
-  child = gtk_bin_get_child (GTK_BIN (widget));
+  child = GTK_WIDGET (ephy_embed_get_web_view (EPHY_EMBED (widget)));
 
   if (child)
     gtk_widget_grab_focus (child);
@@ -306,8 +336,16 @@ static void
 ephy_embed_finalize (GObject *object)
 {
   EphyEmbed *embed = EPHY_EMBED (object);
+  GSList *list;
 
-  g_free (embed->priv->load_failed_uri);
+  g_free (embed->priv->loading_uri);
+
+  list = embed->priv->destroy_on_transition_list;
+  for (; list; list = list->next) {
+    GtkWidget *widget = GTK_WIDGET (list->data);
+    g_signal_handlers_disconnect_by_func (widget, remove_from_destroy_list_cb, embed);
+  }
+  g_slist_free (embed->priv->destroy_on_transition_list);
 
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
 }
@@ -357,49 +395,6 @@ ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
   return TRUE;
 }
 
-static gboolean
-mime_type_policy_decision_requested_cb (WebKitWebView *web_view,
-                                        WebKitWebFrame *frame,
-                                        WebKitNetworkRequest *request,
-                                        const char *mime_type,
-                                        WebKitWebPolicyDecision *decision,
-                                        EphyEmbed *embed)
-{
-  EphyWebViewDocumentType type;
-
-  g_return_val_if_fail (mime_type, FALSE);
-
-  /* Get the mime type for the page only from the main frame */
-  if (webkit_web_view_get_main_frame (web_view) == frame) {
-    type = EPHY_WEB_VIEW_DOCUMENT_OTHER;
-
-    if (!strcmp (mime_type, "text/html"))
-      type = EPHY_WEB_VIEW_DOCUMENT_HTML;
-    else if (!strcmp (mime_type, "application/xhtml+xml"))
-      type = EPHY_WEB_VIEW_DOCUMENT_XML;
-    else if (!strncmp (mime_type, "image/", 6))
-      type = EPHY_WEB_VIEW_DOCUMENT_IMAGE;
-
-    /* FIXME: mayb  e it makes more sense to have an API to query the mime
-     * type when the load of a page starts than doing this here.
-     */
-    /* FIXME: rename ge-document-type (and all ge- signals...) to
-     * something else
-     */
-    g_signal_emit_by_name (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), "ge-document-type", type);
-  }
-
-  /* If WebKit can't handle the mime type start the download
-     process */
-  /* FIXME: need to use ephy_file_check_mime if auto-downloading */
-  if (!webkit_web_view_can_show_mime_type (web_view, mime_type)) {
-    webkit_web_policy_decision_download (decision);
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 static void
 download_requested_dialog_response_cb (GtkDialog *dialog,
                                        int response_id,
@@ -409,9 +404,8 @@ download_requested_dialog_response_cb (GtkDialog *dialog,
     DownloaderView *dview;
     char *uri;
 
-    uri = gtk_file_chooser_get_uri  (GTK_FILE_CHOOSER(dialog));
-    webkit_download_set_destination_uri (download, uri);
-    g_free (uri);
+    uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+    g_object_set_data (G_OBJECT (download), "user-destination-uri", uri);
 
     dview = EPHY_DOWNLOADER_VIEW (ephy_embed_shell_get_downloader_view (embed_shell));
     downloader_view_add_download (dview, download);
@@ -421,6 +415,8 @@ download_requested_dialog_response_cb (GtkDialog *dialog,
   }
 
   gtk_widget_destroy (GTK_WIDGET (dialog));
+  /* User provided us with a destination or cancelled, unfreeze. */
+  g_object_thaw_notify (G_OBJECT (download));
   g_object_unref (download);
 }
 
@@ -574,7 +570,7 @@ define_destination_uri (WebKitDownload *download,
   return TRUE;
 }
 
-static void
+static gboolean
 perform_auto_download (WebKitDownload *download)
 {
   DownloaderView *dview;
@@ -582,13 +578,29 @@ perform_auto_download (WebKitDownload *download)
   if (!define_destination_uri (download, FALSE)) {
     webkit_download_cancel (download);
     ephy_file_delete_uri (webkit_download_get_destination_uri (download));
-    return;
+    return FALSE;
   }
 
   dview = EPHY_DOWNLOADER_VIEW (ephy_embed_shell_get_downloader_view (embed_shell));
 
   g_object_set_data (G_OBJECT(download), "download-action", GINT_TO_POINTER(DOWNLOAD_ACTION_OPEN));
   downloader_view_add_download (dview, download);
+
+  return TRUE;
+}
+
+void
+ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
+{
+  WebKitNetworkRequest *request;
+  WebKitDownload *download;
+
+  request = webkit_network_request_new (url);
+  download = webkit_download_new (request);
+  g_object_unref (request);
+
+  if (perform_auto_download (download))
+    webkit_download_start (download);
 }
 
 static void
@@ -624,6 +636,8 @@ confirm_action_response_cb (GtkWidget *dialog,
       }
       dview = EPHY_DOWNLOADER_VIEW (ephy_embed_shell_get_downloader_view (embed_shell));
       downloader_view_add_download (dview, download);
+      /* User selected "Open", he won't be providing a destination, unfreeze. */
+      g_object_thaw_notify (G_OBJECT (download));
     }
     g_object_unref (download);
     return;
@@ -668,10 +682,10 @@ confirm_action_from_mime (WebKitWebView *web_view,
     if (content_type) {
       mime_description = g_content_type_get_description (content_type);
       helper_app = g_app_info_get_default_for_type (content_type, FALSE);
+      mime_permission = ephy_file_check_mime (content_type);
 
-      if (helper_app) {
+      if (helper_app)
         action = DOWNLOAD_ACTION_OPEN;
-      }
     }
   }
 
@@ -680,6 +694,14 @@ confirm_action_from_mime (WebKitWebView *web_view,
     action = DOWNLOAD_ACTION_OPEN_LOCATION;
   }
 
+  /* Sometimes downloads can have a mime_description but a NULL helper_app
+   * in that case action is never changed so DOWNLOAD_ACTION_DOWNLOAD remains
+   * as action value. This is the same response value as Save as...
+   * button, which is wrong for the Download button.
+   */
+  if (helper_app == NULL)
+    action = DOWNLOAD_ACTION_OPEN_LOCATION;
+
   action_label = (action == DOWNLOAD_ACTION_OPEN) ? GTK_STOCK_OPEN : STOCK_DOWNLOAD;
   suggested_filename = webkit_download_get_suggested_filename (download);
 
@@ -728,9 +750,11 @@ confirm_action_from_mime (WebKitWebView *web_view,
   button = gtk_button_new_with_mnemonic (_("_Save As..."));
   image = gtk_image_new_from_stock (GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
+  gtk_widget_set_can_default (button, TRUE);
   /* don't show the image! see bug #307818 */
   gtk_widget_show (button);
-  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, DOWNLOAD_ACTION_DOWNLOAD);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+                                button, DOWNLOAD_ACTION_DOWNLOAD);
 
   gtk_dialog_add_button (GTK_DIALOG (dialog),
                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
@@ -739,9 +763,9 @@ confirm_action_from_mime (WebKitWebView *web_view,
 
   gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
-  default_response = action == DOWNLOAD_ACTION_NONE
-        ? (int) GTK_RESPONSE_CANCEL
-        : (int) action;
+  default_response = (action == DOWNLOAD_ACTION_NONE) ?
+                     (int) GTK_RESPONSE_CANCEL : (int) action;
+
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), default_response);
 
   g_object_set_data (G_OBJECT (dialog), "webkit-view", web_view);
@@ -759,20 +783,63 @@ download_status_changed_cb (GObject *object,
 {
   WebKitDownload *download = WEBKIT_DOWNLOAD (object);
 
-  g_return_if_fail (webkit_download_get_status (download) == WEBKIT_DOWNLOAD_STATUS_STARTED);
+  if (webkit_download_get_status (download) == WEBKIT_DOWNLOAD_STATUS_FINISHED)
+  {
+    GFile *destination;
+    GFile *temp;
+    char *destination_uri;
+    const char *temp_uri;
 
-  g_signal_handlers_disconnect_by_func (download,
-                                        download_status_changed_cb,
-                                        web_view);
+    temp_uri = webkit_download_get_destination_uri (download);
+    destination_uri = g_object_get_data (G_OBJECT (download),
+                                         "user-destination-uri");
 
-  /* Is auto-download enabled? */
-  if (eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS)) {
-    perform_auto_download (download);
-    return;
+    /* No user-destination-uri is set, hence this is an auto download and we
+     * have nothing else to do.
+     */
+    if (destination_uri == NULL) return;
+
+    temp = g_file_new_for_uri (temp_uri);
+    destination = g_file_new_for_uri (destination_uri);
+
+    ephy_file_switch_temp_file (destination, temp);
+
+    g_object_unref (destination);
+    g_object_unref (temp);
   }
+  else if (webkit_download_get_status (download) == WEBKIT_DOWNLOAD_STATUS_STARTED)
+  {
+    /* Prevent this callback from being called before the user has selected a
+     * destination. It is freed either here or in
+     * download_requested_dialog_response_cb(). Both situations are mutually
+     * exclusive.
+     *
+     * This freeze is removed either here below, in
+     * download_requested_dialog_response_cb() or confirm_action_response_cb().
+     */
+    g_object_freeze_notify (G_OBJECT (download));
 
-  g_object_ref (download); /* balanced in confirm_action_response_cb */
-  confirm_action_from_mime (web_view, download, DOWNLOAD_ACTION_DOWNLOAD);
+    if (eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS)) {
+      perform_auto_download (download);
+      /* User won't select a destination, unfreeze. */
+      g_object_thaw_notify (G_OBJECT (download));
+      return;
+    }
+
+    g_object_ref (download); /* balanced in confirm_action_response_cb */
+    confirm_action_from_mime (web_view, download, DOWNLOAD_ACTION_DOWNLOAD);
+  }
+}
+
+static gboolean
+download_error_cb (WebKitDownload *download,
+                   gint error_code,
+                   gint error_detail,
+                   const gchar *reason,
+                   WebKitWebView *view)
+{
+  /* FIXME: handle download errors and notify the user */
+  return FALSE;
 }
 
 static gboolean
@@ -785,9 +852,12 @@ download_requested_cb (WebKitWebView *web_view,
 
   /* Wait for the request to be sent in all cases, so that we have a
    * response which may contain a suggested filename */
-  g_signal_connect (download, "notify::status",
-                    G_CALLBACK (download_status_changed_cb),
-                    web_view);
+  g_signal_connect_object (download, "notify::status",
+                           G_CALLBACK (download_status_changed_cb),
+                           web_view, 0);
+  g_signal_connect_object (download, "error",
+                           G_CALLBACK (download_error_cb),
+                           web_view, 0);
 
   /* If we are not performing an auto-download, we will ask the user
    * where they want the file to go to; we will start downloading to a
@@ -799,45 +869,37 @@ download_requested_cb (WebKitWebView *web_view,
   return TRUE;
 }
 
-static gboolean
-load_error_cb (WebKitWebView *web_view,
-               WebKitWebFrame *frame,
-               const char *uri,
-               GError *error,
-               EphyEmbed *embed)
-{
-  /* Flag the page as error. We need the flag to check it when
-     receiving COMMITTED status, since for some reason we are getting
-     that when the load fails too */
-  embed->priv->load_failed_uri = g_strdup (uri);
-
-  return FALSE;
-}
-
 static void
 ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed*)object;
+  GtkWidget *scrolled_window;
   WebKitWebView *web_view;
   WebKitWebInspector *inspector;
   GtkWidget *inspector_sw;
 
-  embed->priv = EPHY_EMBED_GET_PRIVATE (embed);
+  embed->priv->top_widgets_vbox = GTK_BOX (gtk_vbox_new (FALSE, 0));
+  gtk_box_pack_start (GTK_BOX (embed), GTK_WIDGET (embed->priv->top_widgets_vbox),
+                      FALSE, FALSE, 0);
+  gtk_widget_show (GTK_WIDGET (embed->priv->top_widgets_vbox));
+
+  scrolled_window = GTK_WIDGET (embed->priv->scrolled_window);
+  gtk_container_add (GTK_CONTAINER (embed), scrolled_window);
+  gtk_widget_show (scrolled_window);
+
   web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
   embed->priv->web_view = web_view;
-  gtk_container_add (GTK_CONTAINER (embed), GTK_WIDGET (web_view));
+  gtk_container_add (GTK_CONTAINER (embed->priv->scrolled_window), GTK_WIDGET (web_view));
   gtk_widget_show (GTK_WIDGET (web_view));
 
   g_object_connect (web_view,
                     "signal::notify::load-status", G_CALLBACK (load_status_changed_cb), embed,
                     "signal::resource-request-starting", G_CALLBACK (resource_request_starting_cb), embed,
                     "signal::hovering-over-link", G_CALLBACK (hovering_over_link_cb), embed,
-                    "signal::mime-type-policy-decision-requested", G_CALLBACK (mime_type_policy_decision_requested_cb), embed,
                     "signal::download-requested", G_CALLBACK (download_requested_cb), embed,
                     "signal::notify::zoom-level", G_CALLBACK (zoom_changed_cb), embed,
                     "signal::notify::title", G_CALLBACK (title_changed_cb), embed,
                     "signal::notify::uri", G_CALLBACK (uri_changed_cb), embed,
-                    "signal::load-error", G_CALLBACK (load_error_cb), embed,
                     NULL);
 
   embed->priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -875,7 +937,11 @@ ephy_embed_constructed (GObject *object)
 static void
 ephy_embed_init (EphyEmbed *embed)
 {
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (embed),
+  embed->priv = EPHY_EMBED_GET_PRIVATE (embed);
+
+  embed->priv->scrolled_window = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
+
+  gtk_scrolled_window_set_policy (embed->priv->scrolled_window,
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 }
 
@@ -894,3 +960,55 @@ ephy_embed_get_web_view (EphyEmbed *embed)
 
   return EPHY_WEB_VIEW (embed->priv->web_view);
 }
+
+/**
+ * ephy_embed_add_top_widget:
+ * @embed: an #EphyEmbed
+ * @widget: a #GtkWidget
+ * @destroy_on_transition: whether the widget be automatically
+ * destroyed on page transitions
+ *
+ * Adds a #GtkWidget to the top of the embed.
+ */
+void
+ephy_embed_add_top_widget (EphyEmbed *embed, GtkWidget *widget, gboolean destroy_on_transition)
+{
+  GSList *list;
+
+  if (destroy_on_transition) {
+    list = embed->priv->destroy_on_transition_list;
+    list = g_slist_prepend (list, widget);
+    embed->priv->destroy_on_transition_list = list;
+
+    g_signal_connect (widget, "destroy", G_CALLBACK (remove_from_destroy_list_cb), embed);
+  }
+
+  gtk_box_pack_end (embed->priv->top_widgets_vbox,
+                    GTK_WIDGET (widget), TRUE, TRUE, 0);
+}
+
+/**
+ * ephy_embed_remove_top_widget:
+ * @embed: an #EphyEmbed
+ * @widget: a #GtkWidget
+ *
+ * Removes an #GtkWidget from the top of the embed. The #GtkWidget
+ * must be have been added using ephy_embed_add_widget(), and not
+ * have been removed by other means. See gtk_container_remove() for
+ * details.
+ */
+void
+ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
+{
+  if (g_slist_find (embed->priv->destroy_on_transition_list, widget)) {
+    GSList *list;
+    g_signal_handlers_disconnect_by_func (widget, remove_from_destroy_list_cb, embed);
+
+    list = embed->priv->destroy_on_transition_list;
+    list = g_slist_remove (list, widget);
+    embed->priv->destroy_on_transition_list = list;
+  }
+
+  gtk_container_remove (GTK_CONTAINER (embed->priv->top_widgets_vbox),
+                        GTK_WIDGET (widget));
+}
