@@ -194,6 +194,7 @@ ephy_location_entry_finalize (GObject *object)
 	EphyLocationEntryPrivate *priv = entry->priv;
 	
 	g_free (priv->saved_text);
+	g_free (priv->lock_stock_id);
 
 	if (priv->favicon != NULL)
 	{
@@ -1252,6 +1253,9 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	g_signal_connect_after (completion, "action-activated",
 				G_CALLBACK (action_activated_after_cb), entry);
 
+	/* FIXME: this works fine, but the favicons seem to be
+	 * added-removed the view constantly, and the visual effect is
+	 * very ugly */
 #if 0
 	cell = gtk_cell_renderer_pixbuf_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
@@ -1635,7 +1639,7 @@ ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
 {
 	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
 
-	entry->priv->lock_stock_id = (char*)stock_id;
+	entry->priv->lock_stock_id = g_strdup (stock_id);
 
 	if (entry->priv->show_lock)
 		gtk_entry_set_icon_from_stock (GTK_ENTRY (entry->priv->entry),
